from ks_api_client import ks_api
from client.TradingInterface.TradingInterface import TradingInterface


class KotakTrading(TradingInterface):
    def __init__(self, api_key: str, api_secret: str, request_token: str, access_token: str,
                 is_session_generated: bool, oauth_code: str):
        self.userId = api_key
        self.password = api_secret
        self.access_token = access_token
        self.consumer_key = request_token
        self.oauth_code = oauth_code
        self.kotak_obj = ks_api.KSTradeApi(access_token=self.access_token, userid=self.userId,
                                           ip="", app_id="", consumer_key=self.consumer_key,
                                           host="https://sbx.kotaksecurities.com/apim")

    def Login(self):
        data = self.kotak_obj.login(password=self.password)
        if 'Success' in data.keys():
            return 'Success'
        else:
            return 'Failed'

    def PlaceOrder(self, qty, transaction_type, trading_symbol, exchange, product_type):
        super().PlaceOrder(qty, transaction_type, trading_symbol, exchange, product_type)

    def Margins(self):
        super().Margins()

    def Positions(self):
        super().Positions()

    def GenerateSession(self, mPin):
        self.kotak_obj.session_2fa(access_code=self.access_token)

    def GetInstruments(self):
        super().GetInstruments()

    def GetHoldings(self):
        pass

    def GetQuote(self, listInstruments, exchange):
        pass

    def GetOrders(self):
        pass

    def GetProfile(self):
        pass

    def GetQuoteNSE(self):
        pass



from client.TradingInterface.TradingInterface import TradingInterface

from client.KiteTrading.KiteTrading import KiteTrading
from client.uTradeBroker.uTradeBroker import UTradeBroker
from client.MasterTrust.MasterTrust import MasterTrustBroker

BROKER_KITE = "KITE"
BROKER_KOTAK = "KOTAK"
BROKER_ICICI = "ICICI"
BROKER_UTRADE = "uTrade"
BROKER_MASTER_TRUST = "MasterTrustBroker"

EXCHANGE_NSE = "NSE"
EXCHANGE_BSE = "BSE"
EXCHANGE_NFO = "NFO"
EXCHANGE_CDS = "CDS"
EXCHANGE_BFO = "BFO"
EXCHANGE_MCX = "MCX"

PRODUCT_MIS = "MIS"
PRODUCT_CNC = "CNC"
PRODUCT_NRML = "NRML"
PRODUCT_CO = "CO"
PRODUCT_BO = "BO"


class TruinvestBroker(TradingInterface):

    def __init__(self, api_key, api_secret, broker, truinvest_secret, is_session_generated=True,
                 access_token: str = "", request_token: str = "", mPin: str = ""):
        self.api_key = api_key
        self.api_secret = api_secret
        self.request_token = request_token
        self.access_token = access_token
        self.broker_name = broker
        self.broker_obj = None
        if self.broker_name == BROKER_KITE:
            self.broker_obj = KiteTrading(self.api_key, self.api_secret, self.request_token,
                                                      self.access_token, is_session_generated,
                                                      truinvest_secret=truinvest_secret)
        elif self.broker_name == BROKER_KOTAK:
            self.broker_obj = None
        elif self.broker_name == BROKER_ICICI:
            self.broker_obj = None
        elif self.broker_name == BROKER_UTRADE:
            self.broker_obj = UTradeBroker(username=self.api_key, password=self.api_secret)
        elif self.broker_name == BROKER_MASTER_TRUST:
            self.broker_obj = MasterTrustBroker(login_id=api_key, password=api_secret,
                                                            truinvest_secret=truinvest_secret)
            self.broker_obj.Login()
        if not is_session_generated:
            self.broker_obj.GenerateSession(mPin=mPin)

    def Login(self) -> str:
        return self.broker_obj.Login()

    def GetTradingClientObj(self):
        return self.broker_obj.GenerateSession()

    def PlaceOrder(self, qty, transaction_type, trading_symbol, exchange, product_type):
        return self.broker_obj.PlaceOrder(qty, transaction_type, trading_symbol, exchange, product_type)

    def Margins(self):
        return self.broker_obj.Margins()

    def Positions(self):
        return self.broker_obj.Positions()

    def Holdings(self):
        return self.broker_obj.Holdings()

    def Orders(self):
        return self.broker_obj.Orders()

    def Profile(self):
        return self.broker_obj.Profile()

    def GenerateSession(self, mPin="123456"):
        return self.broker_obj.GenerateSession(mPin=mPin)

    def GetInstruments(self):
        return self.broker_obj.GetInstruments()

    def GetQuote(self, listInstruments, exchange):
        return self.broker_obj.GetQuote(listInstruments, exchange)

    def GetAvailableMargin(self):
        return self.broker_obj.GetAvailableMargin()



import json

import os
import requests

from client.TradingInterface.TradingInterface import TradingInterface

BASE_URL = "https://masterswift-beta.mastertrust.co.in/api/v1"

ROUTE_LOGIN = "/user/login"

ROUTE_ORDER = "/orders/"

ROUTE_TWOFA = "/user/twofa"

ROUTE_POSITION = "/positions"

TRUINVEST_DEV = "https://devapi.truinvest.ai"

TRUINVEST_PROD = "https://api.truinvest.ai"

TRUINVEST_ROUTE_INSTRUMENTS = "trade/instrument_mapping/?truinvest_secret={}"

TRUINVEST_ROUTE_MARGIN = "user/fetch-margin?broker={}&api_key={}&truinvest_secret={}"


class MasterTrustBroker(TradingInterface):
    def __init__(self, login_id, password, truinvest_secret: str):
        self.login_id = login_id
        self.password = password
        self.two_fa_token = None
        self.access_token = None
        self.margin = 0
        self.truinvest_secret = truinvest_secret
        self.instrument_token_lookup = dict()
        self.GetInstruments()

    def Login(self):
        payload = {
            "login_id": self.login_id,
            "password": self.password,
            "device": "WEB"
        }
        headers = {
            'Content-Type': 'application/json',
            'App-Secret': '1'
        }
        response = requests.request("POST", BASE_URL + ROUTE_LOGIN, headers=headers, json=payload)
        if response.status_code == 200:
            res = response.json()
            print(res)
            self.two_fa_token = res['data']['twofa_token']
            return True
        return False

    def PlaceOrder(self, qty: int, transaction_type: str, trading_symbol: str, exchange: str, product_type: str):
        headers = {
            'X-Authorization-Token': self.access_token,
            'X-Device-Type': "web",
            'Content-Type': 'application/json'
        }
        payload = {
            "exchange": exchange.upper(),
            "order_type": "MARKET",
            "instrument_token": self.instrument_token_lookup[trading_symbol]['instrument_token'],
            "quantity": qty,
            "disclosed_quantity": 0,
            "order_side": transaction_type.upper(),
            "validity": "DAY",
            "product": product_type.upper(),
            "client_id": self.login_id,
            "user_order_id": 10000,
            "market_protection_percentage": 5,
            "device": "WEB"
        }
        response = requests.request("POST", BASE_URL + ROUTE_ORDER, headers=headers, data=json.dumps(payload))
        print(response.json())

    def Margins(self):
        base_url = TRUINVEST_DEV
        if os.environ.get('env') == 'prod':
            base_url = TRUINVEST_PROD
        route = TRUINVEST_ROUTE_MARGIN.format("MASTERTRUST", self.login_id, self.truinvest_secret)
        url = f"{base_url}/{route}"
        response = requests.request("GET", url)

        return {
            "collateral": response.json()['collateral'],
            "cash": response.json()['cash']
        }

    def Positions(self) -> list:
        querystring = {"type": "historical", "client_id": self.login_id}

        headers = {
            'X-Authorization-Token': self.access_token,
            'X-Device-Type': "web"
        }

        response = requests.request("GET", BASE_URL + ROUTE_POSITION, headers=headers, params=querystring)

        positions = []
        for position in response.json()['data']:
            positions.append({'symbol_id': position['instrument_token'],
                              'tradingsymbol': position['trading_symbol'],
                              'quantity': position['net_quantity'],
                              'pnl': 0,
                              'buy_quantity': position['buy_quantity'],
                              'sell_quantity': position['sell_quantity']})

        return positions

    def GenerateSession(self, mPin: str):
        payload = {
            "login_id": self.login_id,
            "twofa": [
                {
                    "question_id": 1,
                    "answer": mPin
                }
            ],
            "twofa_token": self.two_fa_token
        }
        headers = {
            'Content-Type': 'application/json',
            'App-Secret': '1'
        }
        response = requests.request("POST", BASE_URL + ROUTE_TWOFA, headers=headers, json=payload)
        if response.status_code == 200:
            res = response.json()
            self.access_token = res['data']['auth_token']
            print(res)

    def GetInstruments(self):
        base_url = TRUINVEST_DEV
        if os.environ.get('env') == 'prod':
            base_url = TRUINVEST_PROD
        route = TRUINVEST_ROUTE_INSTRUMENTS.format(self.truinvest_secret)
        url = f"{base_url}/{route}"
        response = requests.request("GET", url)
        instruments = []
        if response.status_code == 200:
            instruments = response.json()

        for instrument in instruments:
            self.instrument_token_lookup[instrument['trading_symbol']] = {
                                            'instrument_token': instrument['token'],
                                            # 'lot_size': instrument['lot_size']
            }

    def Holdings(self):
        super().Holdings()

    def GetQuote(self, listInstruments, exchange):
        super().GetQuote(listInstruments, exchange)

    def Orders(self):
        super().Orders()

    def Profile(self):
        super().Profile()

    def GetAvailableMargin(self):
        margin = self.Margins()
        data = {'adhoc_margin': 0,
                'cash': margin['cash'],
                'opening_balance': margin['cash'],
                'live_balance': margin['cash'],
                'collateral': margin['collateral'],
                'intraday_payin': 0}
        return data

    def GetUtilizedMargin(self):
        pass



class TradingInterface:
    def Login(self):
        pass

    def PlaceOrder(self, qty, transaction_type, trading_symbol, exchange, product_type):
        pass

    def Margins(self):
        pass

    def Positions(self):
        pass

    def GenerateSession(self, mPin):
        pass

    def GetInstruments(self):
        pass

    def Holdings(self):
        pass

    def GetQuote(self, listInstruments, exchange):
        pass

    def Orders(self):
        pass

    def Profile(self):
        pass

    def GetAvailableMargin(self):
        pass

    def GetUtilizedMargin(self):
        pass
